@@ -9,9 +9,10 @@ from twisted.test import proto_helpers
 from twisted.internet import defer, error, task
 from twisted.internet.endpoints import TCP4ServerEndpoint
 from twisted.python.failure import Failure
-from twisted.internet.interfaces import IReactorCore, IProtocolFactory, IReactorTCP
+from twisted.internet.interfaces import IReactorCore, IProtocolFactory, IReactorTCP, IListeningPort
 
 from txtorcon import TorControlProtocol, ITorControlProtocol, TorConfig, DEFAULT_VALUE, HiddenService, launch_tor, TCPHiddenServiceEndpoint, TorNotFound
+from txtorcon import torconfig
 
 from txtorcon.util import delete_file_or_tree
 
@@ -70,7 +71,7 @@ class FakeControlProtocol:
     def set_conf(self, *args):
         for i in range(0, len(args), 2):
             self.sets.append((args[i], args[i + 1]))
-        return defer.succeed('OK')
+        return defer.succeed('')
 
     def add_event_listener(self, nm, cb):
         self.events[nm] = cb
@@ -96,16 +97,17 @@ class ConfigTests(unittest.TestCase):
         self.protocol = FakeControlProtocol([])
 
     def test_boolean_parse_error(self):
-        self.protocol.answers.append('config/names=\nfoo Boolean\nOK')
+        self.protocol.answers.append('config/names=\nfoo Boolean')
         self.protocol.answers.append({'foo': 'bar'})
-        TorConfig(self.protocol)
+        cfg = TorConfig(self.protocol)
+        self.assertEqual(cfg.get_type('foo'), torconfig.Boolean)
         errs = self.flushLoggedErrors(ValueError)
         self.assertEqual(len(errs), 1)
         ## dunno if asserting strings in messages is a good idea...
         self.assertTrue('invalid literal' in errs[0].getErrorMessage())
 
     def test_boolean_parser(self):
-        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean\nOK')
+        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean')
         self.protocol.answers.append({'foo': '0'})
         self.protocol.answers.append({'bar': '1'})
         ## FIXME does a Tor controller only ever send "0" and "1" for
@@ -116,7 +118,7 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(conf.bar is True)
 
     def test_boolean_auto_parser(self):
-        self.protocol.answers.append('config/names=\nfoo Boolean+Auto\nbar Boolean+Auto\nbaz Boolean+Auto\nOK')
+        self.protocol.answers.append('config/names=\nfoo Boolean+Auto\nbar Boolean+Auto\nbaz Boolean+Auto')
         self.protocol.answers.append({'foo': '0'})
         self.protocol.answers.append({'bar': '1'})
         self.protocol.answers.append({'baz': 'auto'})
@@ -127,19 +129,19 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(conf.baz is -1)
 
     def test_string_parser(self):
-        self.protocol.answers.append('config/names=\nfoo String\nOK')
+        self.protocol.answers.append('config/names=\nfoo String')
         self.protocol.answers.append({'foo': 'bar'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 'bar')
 
     def test_int_parser(self):
-        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append('config/names=\nfoo Integer')
         self.protocol.answers.append({'foo': '123'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 123)
 
     def test_int_parser_error(self):
-        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append('config/names=\nfoo Integer')
         self.protocol.answers.append({'foo': '123foo'})
         TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
@@ -147,7 +149,7 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_int_parser_error_2(self):
-        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append('config/names=\nfoo Integer')
         self.protocol.answers.append({'foo': '1.23'})
         TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
@@ -155,26 +157,26 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_linelist_parser(self):
-        self.protocol.answers.append('config/names=\nfoo LineList\nOK')
+        self.protocol.answers.append('config/names=\nfoo LineList')
         self.protocol.answers.append({'foo': 'bar\nbaz'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, ['bar', 'baz'])
 
     def test_listlist_parser_with_list(self):
-        self.protocol.answers.append('config/names=\nfoo LineList\nOK')
+        self.protocol.answers.append('config/names=\nfoo LineList')
         self.protocol.answers.append({'foo': [1, 2, 3]})
 
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, ['1', '2', '3'])
 
     def test_float_parser(self):
-        self.protocol.answers.append('config/names=\nfoo Float\nOK')
+        self.protocol.answers.append('config/names=\nfoo Float')
         self.protocol.answers.append({'foo': '1.23'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 1.23)
 
     def test_float_parser_error(self):
-        self.protocol.answers.append('config/names=\nfoo Float\nOK')
+        self.protocol.answers.append('config/names=\nfoo Float')
         self.protocol.answers.append({'foo': '1.23fff'})
         TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
@@ -182,26 +184,26 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_list(self):
-        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append('config/names=\nbing CommaList')
         self.protocol.answers.append({'bing': 'foo,bar,baz'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.config['bing'], ['foo', 'bar', 'baz'])
         # self.assertEqual(conf.bing, ['foo','bar','baz'])
 
     def test_single_list(self):
-        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append('config/names=\nbing CommaList')
         self.protocol.answers.append({'bing': 'foo'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.config['bing'], ['foo'])
 
     def test_multi_list_space(self):
-        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append('config/names=\nbing CommaList')
         self.protocol.answers.append({'bing': 'foo, bar , baz'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.bing, ['foo', 'bar', 'baz'])
 
     def test_descriptor_access(self):
-        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append('config/names=\nbing CommaList')
         self.protocol.answers.append({'bing': 'foo,bar'})
 
         conf = TorConfig(self.protocol)
@@ -222,7 +224,7 @@ class ConfigTests(unittest.TestCase):
         return d
 
     def test_unknown_descriptor(self):
-        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append('config/names=\nbing CommaList')
         self.protocol.answers.append({'bing': 'foo'})
 
         conf = TorConfig(self.protocol)
@@ -233,7 +235,7 @@ class ConfigTests(unittest.TestCase):
             self.assertTrue('foo' in str(e))
 
     def test_invalid_parser(self):
-        self.protocol.answers.append('config/names=\nSomethingExciting NonExistantParserType\nOK')
+        self.protocol.answers.append('config/names=\nSomethingExciting NonExistantParserType')
         TorConfig(self.protocol)
         errs = self.flushLoggedErrors()
         self.assertEqual(len(errs), 1)
@@ -244,7 +246,7 @@ class ConfigTests(unittest.TestCase):
 
     def test_slutty_postbootstrap(self):
         # test that doPostbootstrap still works in "slutty" mode
-        self.protocol.answers.append('config/names=\nORPort Port\nOK')
+        self.protocol.answers.append('config/names=\nORPort Port')
         ## we can't answer right away, or we do all the _do_setup
         ## callbacks before _setup_ is set -- but we need to do an
         ## answer callback after that to trigger this bug
@@ -255,14 +257,14 @@ class ConfigTests(unittest.TestCase):
 
     def test_immediate_bootstrap(self):
         self.protocol.post_bootstrap = None
-        self.protocol.answers.append('config/names=\nfoo Boolean\nOK')
+        self.protocol.answers.append('config/names=\nfoo Boolean')
         self.protocol.answers.append({'foo': '0'})
         conf = TorConfig(self.protocol)
         self.assertTrue('foo' in conf.config)
 
     def test_multiple_orports(self):
         self.protocol.post_bootstrap = None
-        self.protocol.answers.append('config/names=\nOrPort CommaList\nOK')
+        self.protocol.answers.append('config/names=\nOrPort CommaList')
         self.protocol.answers.append({'OrPort': '1234'})
         conf = TorConfig(self.protocol)
         conf.OrPort = ['1234', '4321']
@@ -271,7 +273,7 @@ class ConfigTests(unittest.TestCase):
                                               ('OrPort', '4321')])
 
     def test_set_multiple(self):
-        self.protocol.answers.append('config/names=\nAwesomeKey String\nOK')
+        self.protocol.answers.append('config/names=\nAwesomeKey String')
         self.protocol.answers.append({'AwesomeKey': 'foo'})
 
         conf = TorConfig(self.protocol)
@@ -287,7 +289,7 @@ class ConfigTests(unittest.TestCase):
         self.assertEqual(self.protocol.sets[0], ('AwesomeKey', 'pac man'))
 
     def test_log_double_save(self):
-        self.protocol.answers.append('config/names=\nLog LineList\nFoo String\nOK''')
+        self.protocol.answers.append('config/names=\nLog LineList\nFoo String''')
         self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
         self.protocol.answers.append({'Foo': 'foo'})
         conf = TorConfig(self.protocol)
@@ -304,7 +306,7 @@ class ConfigTests(unittest.TestCase):
         self.assertEqual(self.protocol.sets, [])
 
     def test_set_save_modify(self):
-        self.protocol.answers.append('config/names=\nLog LineList\nOK')
+        self.protocol.answers.append('config/names=\nLog LineList')
         self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
         conf = TorConfig(self.protocol)
 
@@ -316,7 +318,7 @@ class ConfigTests(unittest.TestCase):
         self.assertTrue(conf.needs_save())
 
     def test_proper_sets(self):
-        self.protocol.answers.append('config/names=\nLog LineList\nOK')
+        self.protocol.answers.append('config/names=\nLog LineList')
         self.protocol.answers.append({'Log': 'foo'})
 
         conf = TorConfig(self.protocol)
@@ -332,7 +334,7 @@ class LogTests(unittest.TestCase):
 
     def setUp(self):
         self.protocol = FakeControlProtocol([])
-        self.protocol.answers.append('config/names=\nLog LineList\nOK''')
+        self.protocol.answers.append('config/names=\nLog LineList''')
         self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
 
     def test_log_set(self):
@@ -467,14 +469,16 @@ class CreateTorrcTests(unittest.TestCase):
         config.Log = ['80 127.0.0.1:80', '90 127.0.0.1:90']
         config.save()
         torrc = config.create_torrc()
-        self.assertEqual(torrc, '''HiddenServiceDir /some/dir
+        lines = torrc.split('\n')
+        lines.sort()
+        torrc = '\n'.join(lines).strip()
+        self.assertEqual(torrc, '''HiddenServiceAuthorizeClient auth
+HiddenServiceDir /some/dir
 HiddenServicePort 80 127.0.0.1:1234
 HiddenServiceVersion 2
-HiddenServiceAuthorizeClient auth
 Log 80 127.0.0.1:80
 Log 90 127.0.0.1:90
-SocksPort 1234
-''')
+SocksPort 1234''')
 
 
 class HiddenServiceTests(unittest.TestCase):
@@ -484,8 +488,7 @@ class HiddenServiceTests(unittest.TestCase):
         self.protocol.answers.append('''config/names=
 HiddenServiceOptions Virtual
 HiddenServiceVersion Dependant
-HiddenServiceAuthorizeClient Dependant
-OK''')
+HiddenServiceAuthorizeClient Dependant''')
 
     def test_options_hidden(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\nHiddenServicePort=80 127.0.0.1:1234\n')
@@ -499,6 +502,7 @@ OK''')
         conf.hiddenservices[0].ports.append('443 127.0.0.1:443')
         self.assertTrue(conf.needs_save())
         conf.save()
+        self.assertEqual(conf.get_type('HiddenServices'), HiddenService)
 
         self.assertEqual(len(self.protocol.sets), 7)
         self.assertEqual(self.protocol.sets[0], ('HiddenServiceDir', '/fake/path'))
@@ -822,7 +826,7 @@ class LaunchTorTests(unittest.TestCase):
         react.advance(timeout + 1)
 
         self.assertTrue(d.called)
-        self.assertTrue(d.result.getErrorMessage().strip().endswith('Tor was killed (KILL).'))
+        self.assertTrue(d.result.getErrorMessage().strip().endswith('Tor was killed (TERM).'))
         return self.assertFailure(d, RuntimeError)
 
     def test_launch_with_timeout_that_doesnt_expire(self):
@@ -887,9 +891,11 @@ class LaunchTorTests(unittest.TestCase):
         trans.protocol = self.protocol
         self.othertrans = trans
         creator = functools.partial(connector, self.protocol, self.transport)
-        d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator, tor_binary='/bin/echo')
+        d = launch_tor(config, FakeReactor(self, trans, on_protocol),
+                       connection_creator=creator, tor_binary='/bin/echo')
         d.addCallback(self.fail)        # should't get callback
         d.addErrback(self.setup_fails_stderr)
+        self.assertFalse(self.protocol.on_disconnect)
         return d
 
     def test_tor_connection_fails(self):
@@ -1065,12 +1071,13 @@ class FakeProtocolFactory:
 
 
 class FakeListeningPort(object):
+    implements(IListeningPort)
 
     def startListening(self):
-        print "startListening"
+        self.factory.doStart()
 
     def stopListening(self):
-        print "stopListening"
+        self.factory.doStop()
 
     def getHost(self):
         return "host"
@@ -1086,11 +1093,10 @@ class FakeReactorTcp(object):
             self.failures -= 1
             raise error.CannotListenError(None, None, None)
 
-        return FakeListeningPort()
-
-    def connectTCP(self, host, port, factory, **kwargs):
-        print "listenTCP", port, factory, kwargs
-        return FakeListeningPort()
+        p = FakeListeningPort()
+        p.factory = factory
+        p.startListening()
+        return p
 
 
 class EndpointTests(unittest.TestCase):
@@ -1103,12 +1109,38 @@ class EndpointTests(unittest.TestCase):
     def test_basic(self):
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
         self.protocol.answers.append('HiddenServiceOptions')
         self.config.bootstrap()
         self.assertEqual('127.0.0.1', ep.tcp_endpoint._interface)
+        ## make sure _ListWrapper's __repr__ doesn't explode
+        repr(self.config.HiddenServices)
         return d
 
+    def test_multiple_listen(self):
+        ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
+        d0 = ep.listen(FakeProtocolFactory())
+
+        @defer.inlineCallbacks
+        def more_listen(arg):
+            yield arg.stopListening()
+            d1 = ep.listen(FakeProtocolFactory())
+
+            def foo(arg):
+                return arg
+            d1.addBoth(foo)
+            defer.returnValue(arg)
+            return
+        d0.addBoth(more_listen)
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
+        self.protocol.answers.append('HiddenServiceOptions')
+        self.config.bootstrap()
+
+        def check(arg):
+            self.assertEqual('127.0.0.1', ep.tcp_endpoint._interface)
+        d0.addCallback(check).addErrback(self.fail)
+        return d0
+
     def test_bad_listener(self):
         def test_gen(*args, **kw):
             kw['interface'] = '0.0.0.0'
@@ -1127,7 +1159,7 @@ class EndpointTests(unittest.TestCase):
         d.addErrback(error_cb)
 
         ## enough answers so the config bootstraps properly
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
         self.protocol.answers.append('HiddenServiceOptions')
         self.config.bootstrap()
 
@@ -1144,8 +1176,7 @@ class EndpointTests(unittest.TestCase):
 
     def test_already_bootstrapped(self):
         self.protocol.answers.append('''config/names=
-HiddenServiceOptions Virtual
-OK''')
+HiddenServiceOptions Virtual''')
         self.protocol.answers.append('HiddenServiceOptions')
 
         self.config.bootstrap()
@@ -1158,7 +1189,7 @@ OK''')
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123, '/mumble/mumble')
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
         self.protocol.answers.append('HiddenServiceOptions')
 
         self.config.bootstrap()
@@ -1185,7 +1216,7 @@ OK''')
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
         self.protocol.answers.append('HiddenServiceOptions')
         self.config.bootstrap()
         return d
@@ -1199,7 +1230,7 @@ OK''')
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
         self.protocol.answers.append('HiddenServiceOptions')
         self.config.bootstrap()
         d.addErrback(self.check_error)
@@ -1207,7 +1238,6 @@ OK''')
 
 
 class ErrorTests(unittest.TestCase):
-
     def test_no_tor_binary(self):
         """FIXME: do I really need all this crap in here?"""
         from txtorcon import torconfig
