@@ -1,9 +1,9 @@
 from __future__ import with_statement
 
-from twisted.python import log
+from twisted.python import log, failure
 from twisted.trial import unittest
 from twisted.test import proto_helpers
-from twisted.internet import defer
+from twisted.internet import defer, error
 
 from txtorcon import TorControlProtocol, TorProtocolFactory, TorState
 from txtorcon import ITorControlProtocol
@@ -33,7 +33,6 @@ class CallbackChecker:
 
 class InterfaceTests(unittest.TestCase):
     def test_implements(self):
-        from zope.interface import implements, implementedBy
         self.assertTrue(ITorControlProtocol.implementedBy(TorControlProtocol))
 
     def test_object_implements(self):
@@ -136,6 +135,47 @@ class AuthenticationTests(unittest.TestCase):
         self.assertTrue(self.auth_failed)
 
 
+class DisconnectionTests(unittest.TestCase):
+    def setUp(self):
+        self.protocol = TorControlProtocol()
+        self.protocol.connectionMade = lambda: None
+        self.transport = proto_helpers.StringTransportWithDisconnection()
+        self.protocol.makeConnection(self.transport)
+        ## why doesn't makeConnection do this?
+        self.transport.protocol = self.protocol
+
+    def tearDown(self):
+        self.protocol = None
+
+    def test_disconnect_callback(self):
+        """
+        see that we get our callback on_disconnect if the transport
+        goes away
+        """
+        def it_was_called(*args):
+            it_was_called.yes = True
+            return None
+        it_was_called.yes = False
+        self.protocol.on_disconnect.addCallback(it_was_called).addErrback(it_was_called)
+        f = failure.Failure(error.ConnectionDone("It's all over"))
+        self.protocol.connectionLost(f)
+        self.assertTrue(it_was_called.yes)
+
+    def test_disconnect_errback(self):
+        """
+        see that we get our callback on_disconnect if the transport
+        goes away
+        """
+        def it_was_called(*args):
+            it_was_called.yes = True
+            return None
+        it_was_called.yes = False
+        self.protocol.on_disconnect.addCallback(it_was_called).addErrback(it_was_called)
+        f = failure.Failure(RuntimeError("The thing didn't do the stuff."))
+        self.protocol.connectionLost(f)
+        self.assertTrue(it_was_called.yes)
+
+
 class ProtocolTests(unittest.TestCase):
 
     def setUp(self):
@@ -307,8 +347,10 @@ OK''' % cookietmp.name)
         return d
 
     def test_async(self):
-        ## test the example from control-spec.txt to see that we
-        ## handle interleaved async notifications properly.
+        """
+        test the example from control-spec.txt to see that we
+        handle interleaved async notifications properly.
+        """
         self.protocol._set_valid_events('CIRC')
         self.protocol.add_event_listener('CIRC', lambda _: None)
         self.send("250 OK")
@@ -354,7 +396,7 @@ OK''' % cookietmp.name)
         return d
 
     def incremental_check(self, expected, actual):
-        if '=' in actual or actual == 'OK':
+        if '=' in actual:
             return
         self.assertEqual(expected, actual)
 
@@ -375,6 +417,11 @@ OK''' % cookietmp.name)
         self.send("250 OK")
         return d
 
+    def test_getinfo_one_line(self):
+        d = self.protocol.get_info("foo", functools.partial(self.incremental_check, "bar"))
+        self.send('250 foo=bar')
+        return d
+
     def test_getconf(self):
         d = self.protocol.get_conf("SOCKSPORT ORPORT")
         d.addCallback(CallbackChecker({'SocksPort': '9050', 'ORPort': '0'}))
@@ -390,7 +437,7 @@ OK''' % cookietmp.name)
         return d
 
     def response_ok(self, v):
-        self.assertEqual(v, 'OK')
+        self.assertEqual(v, '')
 
     def test_setconf(self):
         d = self.protocol.set_conf("foo", "bar").addCallback(functools.partial(self.response_ok))
@@ -578,7 +625,7 @@ OK''' % cookietmp.name)
             self.assertTrue('FOO' in str(e))
 
     def checkContinuation(self, v):
-        self.assertEqual(v, "key=\nvalue0\nvalue1\nOK")
+        self.assertEqual(v, "key=\nvalue0\nvalue1")
 
     def test_continuationLine(self):
         d = self.protocol.get_info_raw("key")
@@ -605,8 +652,7 @@ OK''' % cookietmp.name)
 r fake YkkmgCNRV1/35OPWDvo7+1bmfoo tanLV/4ZfzpYQW0xtGFqAa46foo 2011-12-12 16:29:16 12.45.56.78 443 80
 s Exit Fast Guard HSDir Named Running Stable V2Dir Valid
 w Bandwidth=518000
-p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863,2082-2083,2086-2087,2095-2096,3128,4321,5050,5190,5222-5223,6679,6697,7771,8000,8008,8080-8081,8090,8118,8123,8181,8300,8443,8888
-OK"""))
+p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863,2082-2083,2086-2087,2095-2096,3128,4321,5050,5190,5222-5223,6679,6697,7771,8000,8008,8080-8081,8090,8118,8123,8181,8300,8443,8888"""))
 
         self.send("250+ns/id/624926802351575FF7E4E3D60EFA3BFB56E67E8A=")
         self.send("r fake YkkmgCNRV1/35OPWDvo7+1bmfoo tanLV/4ZfzpYQW0xtGFqAa46foo 2011-12-12 16:29:16 12.45.56.78 443 80")
@@ -639,7 +685,7 @@ class ParseTests(unittest.TestCase):
         self.controller.connectionMade = lambda _: None
 
     def test_keywords(self):
-        x = parse_keywords('events/names=CIRC STREAM ORCONN BW DEBUG INFO NOTICE WARN ERR NEWDESC ADDRMAP AUTHDIR_NEWDESCS DESCCHANGED NS STATUS_GENERAL STATUS_CLIENT STATUS_SERVER GUARD STREAM_BW CLIENTS_SEEN NEWCONSENSUS BUILDTIMEOUT_SET\nOK')
+        x = parse_keywords('events/names=CIRC STREAM ORCONN BW DEBUG INFO NOTICE WARN ERR NEWDESC ADDRMAP AUTHDIR_NEWDESCS DESCCHANGED NS STATUS_GENERAL STATUS_CLIENT STATUS_SERVER GUARD STREAM_BW CLIENTS_SEEN NEWCONSENSUS BUILDTIMEOUT_SET')
         self.assertTrue('events/names' in x)
         self.assertEqual(x['events/names'], 'CIRC STREAM ORCONN BW DEBUG INFO NOTICE WARN ERR NEWDESC ADDRMAP AUTHDIR_NEWDESCS DESCCHANGED NS STATUS_GENERAL STATUS_CLIENT STATUS_SERVER GUARD STREAM_BW CLIENTS_SEEN NEWCONSENSUS BUILDTIMEOUT_SET')
         self.assertEqual(len(x.keys()), 1)
@@ -751,8 +797,7 @@ s Fast Guard HSDir Named Running Stable V2Dir Valid
 w Bandwidth=18700
 p reject 1-65535""")
         self.controller._circuit_status("""circuit-status=
-4472 BUILT $FF1003D2D14B4B9D03933F8EDFBC46C952E82A59=Tecumseh,$C185D4A4B069CD559FCD548C8063B475385D777F=l0l,$7FE4F2FFE07A96062BD0DB5B7FAECEFCBD8CF192=wildnl PURPOSE=GENERAL
-""")
+4472 BUILT $FF1003D2D14B4B9D03933F8EDFBC46C952E82A59=Tecumseh,$C185D4A4B069CD559FCD548C8063B475385D777F=l0l,$7FE4F2FFE07A96062BD0DB5B7FAECEFCBD8CF192=wildnl PURPOSE=GENERAL""")
         self.assertEqual(len(self.controller.circuits), 1)
         self.assertTrue(4472 in self.controller.circuits)
 
