@@ -171,6 +171,7 @@ class FakeControlProtocol:
     def __init__(self):
         self.is_owned = None
         self.post_bootstrap = defer.succeed(self)
+        self.on_disconnect = defer.Deferred()
 
 
 class InternalMethodsTests(unittest.TestCase):
@@ -288,7 +289,7 @@ class StateTests(unittest.TestCase):
 
     def test_single_streams(self):
         self.state.circuits[496] = FakeCircuit(496)
-        self.state._stream_status('stream-status=123 SUCCEEDED 496 www.example.com:6667\r\nOK')
+        self.state._stream_status('stream-status=123 SUCCEEDED 496 www.example.com:6667')
         self.assertEqual(len(self.state.streams), 1)
 
     def send(self, line):
@@ -589,33 +590,39 @@ class StateTests(unittest.TestCase):
         print self.transport.value()
         self.assertEqual(self.transport.value(), '')
 
-    def test_close_stream(self):
+    def test_close_stream_with_id(self):
         stream = Stream(self.state)
         stream.id = 1
-        try:
-            self.state.close_stream(stream)
-            self.assertTrue(False)
-        except KeyError:
-            pass
 
         self.state.streams[1] = stream
         self.state.close_stream(stream)
         self.assertEqual(self.transport.value(), 'CLOSESTREAM 1 1\r\n')
 
+    def test_close_stream_with_stream(self):
+        stream = Stream(self.state)
+        stream.id = 1
+
+        self.state.streams[1] = stream
+        self.state.close_stream(stream.id)
+        self.assertEqual(self.transport.value(), 'CLOSESTREAM 1 1\r\n')
+
     def test_close_stream_invalid_reason(self):
         stream = Stream(self.state)
         stream.id = 1
         self.state.streams[1] = stream
         self.assertRaises(ValueError, self.state.close_stream, stream, 'FOO_INVALID_REASON')
 
-    def test_close_circuit(self):
+    def test_close_circuit_with_id(self):
+        circuit = Circuit(self.state)
+        circuit.id = 1
+
+        self.state.circuits[1] = circuit
+        self.state.close_circuit(circuit.id)
+        self.assertEqual(self.transport.value(), 'CLOSECIRCUIT 1\r\n')
+
+    def test_close_circuit_with_circuit(self):
         circuit = Circuit(self.state)
         circuit.id = 1
-        try:
-            self.state.close_circuit(circuit)
-            self.assertTrue(False)
-        except KeyError:
-            pass
 
         self.state.circuits[1] = circuit
         self.state.close_circuit(circuit)
@@ -624,14 +631,14 @@ class StateTests(unittest.TestCase):
     def test_close_circuit_with_flags(self):
         circuit = Circuit(self.state)
         circuit.id = 1
-        try:
-            self.state.close_circuit(circuit, IfUnused=True)
-            self.assertTrue(False)
-        except KeyError:
-            pass
+        # try:
+        #     self.state.close_circuit(circuit.id, IfUnused=True)
+        #     self.assertTrue(False)
+        # except KeyError:
+        #     pass
 
         self.state.circuits[1] = circuit
-        self.state.close_circuit(circuit, IfUnused=True)
+        self.state.close_circuit(circuit.id, IfUnused=True)
         self.assertEqual(self.transport.value(), 'CLOSECIRCUIT 1 IfUnused\r\n')
 
     def test_circuit_destroy(self):
@@ -955,29 +962,6 @@ s Fast Guard Running Stable Valid
         self.assertTrue('Unnamed' in self.state.routers)
         self.assertTrue('$00126582E505CF596F412D23ABC9E14DD4625C49' in self.state.routers)
 
-    def test_newdesc_parse(self):
-        """
-        should this mostly go in test_router instead? all we need to
-        confirm about the TorState class is that it sends the right
-        GETINFO. Well, we're also testing the args get split up
-        properly and so forth.
-        """
-        self.state._newdesc_update("$624926802351575FF7E4E3D60EFA3BFB56E67E8A=fake CLOSED REASON=IOERROR")
-
-        # TorState should issue "GETINFO ns/id/624926802351575FF7E4E3D60EFA3BFB56E67E8A"
-        # because it hasn't seen this yet, and we'll answer to see if it updates properly
-        d = self.protocol.defer
-        d.addCallback(self.confirm_router_state)
-        self.send("250+ns/id/624926802351575FF7E4E3D60EFA3BFB56E67E8A=")
-        self.send("r fake YkkmgCNRV1/35OPWDvo7+1bmfoo tanLV/4ZfzpYQW0xtGFqAa46foo 2011-12-12 16:29:16 12.45.56.78 443 80")
-        self.send("s Exit Fast Guard HSDir Named Running Stable V2Dir Valid FutureProof")
-        self.send("w Bandwidth=518000")
-        self.send("p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863,2082-2083,2086-2087,2095-2096,3128,4321,5050,5190,5222-5223,6679,6697,7771,8000,8008,8080-8081,8090,8118,8123,8181,8300,8443,8888")
-        self.send(".")
-        self.send("250 OK")
-
-        return d
-
     def test_stream_create(self):
         self.state._stream_update('1610 NEW 0 1.2.3.4:56')
         self.assertTrue(1610 in self.state.streams)
