@@ -1,9 +1,15 @@
+import time
+import datetime
+
 from twisted.python import log
 from twisted.internet import defer
 from interface import IRouterContainer
 
 from txtorcon.util import find_keywords
 
+#look like "2014-01-25T02:12:14.593772"
+TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
+
 
 class Circuit(object):
     """
@@ -70,11 +76,26 @@ class Circuit(object):
         self.id = None
         self.state = 'UNKNOWN'
         self.build_flags = []
+        self.flags = {}
 
         ## this is used to hold a Deferred that will callback() when
         ## this circuit is being CLOSED or FAILED.
         self._closing_deferred = None
 
+        ## caches parsed value for time_created()
+        self._time_created = None
+
+    @property
+    def time_created(self):
+        if self._time_created is not None:
+            return self._time_created
+        if 'TIME_CREATED' in self.flags:
+            ## strip off milliseconds
+            t = self.flags['TIME_CREATED'].split('.')[0]
+            tstruct = time.strptime(t, TIME_FORMAT)
+            self._time_created = datetime.datetime(*tstruct[:7])
+        return self._time_created
+
     def listen(self, listener):
         if listener not in self.listeners:
             self.listeners.append(listener)
@@ -102,10 +123,21 @@ class Circuit(object):
 
         def close_command_is_queued(*args):
             return self._closing_deferred
-        d = self.torstate.close_circuit(self, **kw)
+        d = self.torstate.close_circuit(self.id, **kw)
         d.addCallback(close_command_is_queued)
         return self._closing_deferred
 
+    def age(self, now=datetime.datetime.now()):
+        """
+        Returns an integer which is the difference in seconds from
+        'now' to when this circuit was created.
+
+        Returns None if there is no created-time.
+        """
+        if not self.time_created:
+            return None
+        return (now - self.time_created).seconds
+
     def _create_flags(self, kw):
         "this clones the kw dict, adding a lower-case version of every key (duplicated in stream.py; put in util?)"
 
@@ -127,6 +159,7 @@ class Circuit(object):
         self.state = args[1]
 
         kw = find_keywords(args)
+        self.flags = kw
         if 'PURPOSE' in kw:
             self.purpose = kw['PURPOSE']
         if 'BUILD_FLAGS' in kw:
@@ -144,6 +177,9 @@ class Circuit(object):
 
         elif self.state == 'CLOSED':
             if len(self.streams) > 0:
+                ## FIXME it seems this can/does happen if a remote
+                ## router crashes or otherwise shuts down a circuit
+                ## with streams on it still
                 log.err(RuntimeError("Circuit is %s but still has %d streams" %
                                      (self.state, len(self.streams))))
             flags = self._create_flags(kw)
