@@ -161,14 +161,6 @@ class TCPHiddenServiceEndpoint(object):
         self.config.HiddenServices.append(self.hiddenservice)
         return arg
 
-    def _do_error(self, f):
-        """
-        handle errors. FIXME
-        """
-
-        print "ERROR", f
-        return f
-
     def listen(self, protocolfactory):
         """
         Implement :api:`twisted.internet.interfaces.IStreamServerEndpoint <IStreamServerEndpoint>`.
@@ -184,15 +176,21 @@ class TCPHiddenServiceEndpoint(object):
         """
 
         self.protocolfactory = protocolfactory
-        if self.config.post_bootstrap:
-            d = self.config.post_bootstrap.addCallback(self._create_hiddenservice).addErrback(self._do_error)
+        if self.hiddenservice is None:
+            ## we don't have a hidden service yet, but if the config
+            ## isn't bootstrapped, we need to wait for it first
+            if self.config.post_bootstrap:
+                d = self.config.post_bootstrap.addCallback(self._create_hiddenservice)
 
-        elif self.hiddenservice is None:
-            self._create_hiddenservice(None)
-            d = self.config.save()
+            else:
+                self._create_hiddenservice(None)
+                d = self.config.save()
 
         else:
-            raise RuntimeError("FIXME")
+            ## we already have a hidden service created, but still
+            ## want a Deferred so the _create_listener flow is the
+            ## same
+            d = defer.succeed(self)
 
         d.addCallback(self._create_listener).addErrback(self._retry_local_port)
         return d
@@ -350,7 +348,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         """
 
         try:
-            self.transport.signalProcess('KILL')
+            self.transport.signalProcess('TERM')
         except error.ProcessExitedAlready:
             self.transport.loseConnection()
         self._did_timeout = True
@@ -379,7 +377,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
 
         self.cleanup()
 
-        if isinstance(status.value, error.ProcessDone):
+        if isinstance(status.value, error.ProcessDone) and not self._did_timeout:
             return
 
         if status.value.exitCode is None:
@@ -936,19 +934,11 @@ class TorConfig(object):
         ``things which might get into the running Tor if save() were
         to be called''
         """
-        if name.startswith('__') and name.endswith('__'):
-            # Special case __foobar__ attributes to go for the real dict
-            # instead of self.config and raise AttributeError instead
-            # of KeyError
-            try:
-                return self.__dict__[name]
-            except KeyError, e:
-                raise AttributeError(str(e))
-        else:
-            rn = self._find_real_name(name)
-            if '_slutty_' in self.__dict__ and rn in self.unsaved:
-                return self.unsaved[rn]
-            return self.config[rn]
+
+        rn = self._find_real_name(name)
+        if '_slutty_' in self.__dict__ and rn in self.unsaved:
+            return self.unsaved[rn]
+        return self.config[rn]
 
     def get_type(self, name):
         """
@@ -1077,7 +1067,7 @@ class TorConfig(object):
     @defer.inlineCallbacks
     def _do_setup(self, data):
         for line in data.split('\n'):
-            if line == "config/names=" or line == "OK":
+            if line == "config/names=":
                 continue
 
             (name, value) = line.split()
