@@ -3,8 +3,9 @@ from __future__ import with_statement
 from twisted.python import log
 from twisted.internet import defer
 from twisted.internet.interfaces import IProtocolFactory
+from twisted.internet.error import ConnectionDone
 from twisted.protocols.basic import LineOnlyReceiver
-from zope.interface import implements, implementedBy
+from zope.interface import implements
 
 from txtorcon.util import hmac_sha256, compare_via_hash
 from txtorcon.log import txtorlog
@@ -223,6 +224,12 @@ class TorControlProtocol(LineOnlyReceiver):
         self.valid_signals = []
         """A list of all valid signals we accept from Tor"""
 
+        self.on_disconnect = defer.Deferred()
+        """
+        This Deferred is triggered when the connection is closed. If
+        there was an error, the errback is called instead.
+        """
+
         self.post_bootstrap = defer.Deferred()
         """
         This Deferred is triggered when we're done setting up
@@ -315,7 +322,10 @@ class TorControlProtocol(LineOnlyReceiver):
         See :meth:`getinfo <txtorcon.TorControlProtocol.get_info>`
         """
 
-        return self.queue_command('GETINFO %s' % key, line_cb)
+        def strip_ok_and_call(line):
+            if line.strip() != 'OK':
+                line_cb(line)
+        return self.queue_command('GETINFO %s' % key, strip_ok_and_call)
 
     ## The following methods are the main TorController API and
     ## probably the most interesting for users.
@@ -338,7 +348,7 @@ class TorControlProtocol(LineOnlyReceiver):
             in the callback chain; if you want to avoid the parsing
             into a dict, you can use get_info_raw instead.
         """
-        return self.get_info_raw(*args).addCallback(parse_keywords).addErrback(log.err)
+        return self.get_info_raw(*args).addCallback(parse_keywords)
 
     def get_conf(self, *args):
         """
@@ -427,7 +437,9 @@ class TorControlProtocol(LineOnlyReceiver):
 
         :Return: ``None``
 
-        .. todo:: need an interface for the callback
+        .. todo::
+            need an interface for the callback
+            show how to tie in Stem parsing if you want
         """
 
         if not evt in self.valid_events.values():
@@ -506,10 +518,21 @@ class TorControlProtocol(LineOnlyReceiver):
         self.fsm.process(line)
 
     def connectionMade(self):
-        "LineOnlyReceiver API (or parent?)"
+        "Protocol API"
         txtorlog.msg('got connection, authenticating')
         self.protocolinfo().addCallback(self._do_authenticate).addErrback(self._auth_failed)
 
+    def connectionLost(self, reason):
+        "Protocol API"
+        txtorlog.msg('connection terminated: ' + str(reason))
+        if self.on_disconnect.callbacks:
+            if reason.check(ConnectionDone):
+                self.on_disconnect.callback(self)
+            else:
+                self.on_disconnect.errback(reason)
+        self.on_disconnect = None
+        return None
+
     def _handle_notify(self, code, rest):
         """
         Internal method to deal with 600-level responses.
@@ -761,6 +784,8 @@ class TorControlProtocol(LineOnlyReceiver):
         if self.code >= 200 and self.code < 300:
             if self.defer is None:
                 raise RuntimeError("Got a response, but didn't issue a command.")
+            if resp.endswith('\nOK'):
+                resp = resp[:-3]
             self.defer.callback(resp)
         elif self.code >= 500 and self.code < 600:
             err = TorProtocolError(self.code, resp)
