@@ -172,7 +172,7 @@ class TorState(object):
 
     def __init__(self, protocol, bootstrap=True, write_state_diagram=False):
         self.protocol = ITorControlProtocol(protocol)
-        self.protocol.connectionLost = self.connection_lost
+        ## fixme could use protocol.on_disconnect to re-connect; see issue #3
 
         ## could override these to get your own Circuit/Stream subclasses
         ## to track these things
@@ -308,9 +308,6 @@ class TorState(object):
         self._router.policy = args[1:]
         self._router = None
 
-    def connection_lost(self, *args):
-        pass
-
     @defer.inlineCallbacks
     def _bootstrap(self, arg=None):
         "This takes an arg so we can use it as a callback (see __init__)."
@@ -339,11 +336,10 @@ class TorState(object):
         am = yield self.protocol.get_info_raw(key)
         ## strip addressmappsings/all= and OK\n from raw data
         am = am[len(key) + 1:]
-        if am.strip() != 'OK':
-            for line in am.split('\n')[:-1]:
-                if len(line.strip()) == 0:
-                    continue            # FIXME
-                self.addrmap.update(line)
+        for line in am.split('\n'):
+            if len(line.strip()) == 0:
+                continue            # FIXME
+            self.addrmap.update(line)
 
         self._add_events()
 
@@ -448,8 +444,9 @@ class TorState(object):
         (e.g. "SomeFlag=True"). Currently there are none that Tor accepts.
         """
 
-        if stream.id not in self.streams:
-            raise KeyError("No such stream: %d" % stream.id)
+        if type(stream) != int:
+            ## assume it's a Stream instance
+            stream = stream.id
         try:
             reason = int(reason)
         except ValueError:
@@ -460,10 +457,11 @@ class TorState(object):
 
         flags = flags_from_dict(kwargs)
 
-        cmd = 'CLOSESTREAM %s %d%s' % (str(stream.id), reason, flags)
+        ## stream is now an ID no matter what we passed in
+        cmd = 'CLOSESTREAM %d %d%s' % (stream, reason, flags)
         return self.protocol.queue_command(cmd)
 
-    def close_circuit(self, circ, **kwargs):
+    def close_circuit(self, circid, **kwargs):
         """
         This sends a CLOSECIRCUIT command, using any keyword arguments
         passed as the Flags (currently, that is just 'IfUnused' which
@@ -476,10 +474,11 @@ class TorState(object):
         <txtorcon.circuit.Circuit.close>`
         """
 
-        if circ.id not in self.circuits:
-            raise KeyError("No such circuit: %d" % circ.id)
+        if type(circid) != int:
+            ## assume it's a Circuit instance
+            circid = circid.id
         flags = flags_from_dict(kwargs)
-        return self.protocol.queue_command('CLOSECIRCUIT %s%s' % (circ.id, flags))
+        return self.protocol.queue_command('CLOSECIRCUIT %s%s' % (circid, flags))
 
     def add_circuit_listener(self, icircuitlistener):
         listen = ICircuitListener(icircuitlistener)
@@ -525,8 +524,17 @@ class TorState(object):
         else:
             if routers[0] not in self.entry_guards.values():
                 warnings.warn("Building a circuit not starting with a guard: %s" % (str(routers),), RuntimeWarning)
-            cmd = "EXTENDCIRCUIT 0 " + ','.join(map(lambda x: x.id_hex[1:], routers))
-
+            cmd = "EXTENDCIRCUIT 0 "
+            first = True
+            for router in routers:
+                if first:
+                    first = False
+                else:
+                    cmd += ','
+                if isinstance(router, types.StringType) and len(router) == 40 and hashFromHexId(router):
+                    cmd += router
+                else:
+                    cmd += router.id_hex[1:]
         d = self.protocol.queue_command(cmd)
         d.addCallback(self._find_circuit_after_extend)
         return d
@@ -587,7 +595,7 @@ class TorState(object):
     def _circuit_status(self, data):
         """Used internally as a callback for updating Circuit information"""
 
-        data = data[len('circuit-status='):].split('\n')[:-1]
+        data = data[len('circuit-status='):].split('\n')
         ## sometimes there's a newline after circuit-status= and
         ## sometimes not, so we get rid of it
         if len(data) and len(data[0].strip()) == 0:
@@ -604,7 +612,7 @@ class TorState(object):
         # single-stream case which has "stream-status=123 blahblah"
         # (i.e. the key + value on one line)
 
-        lines = data.split('\n')[:-1]
+        lines = data.split('\n')
         if len(lines) == 1:
             d = lines[0][len('stream-status='):]
             # if there are actually 0 streams, then there's nothing
@@ -632,21 +640,6 @@ class TorState(object):
 
         txtorlog.msg(len(self.guards), "GUARDs")
 
-    def _newdesc_update(self, args):
-        """
-        Callback used internall for ORCONN and NEWDESC events to
-        update Router information.
-
-        FIXME: need to look at state for NEWDESC; if it's CLOSED we
-        probably want to remove it from dicts...
-        """
-
-        hsh = args[:41]
-        if hsh not in self.routers:
-            txtorlog.msg("haven't seen", hsh, "yet!")
-        self.protocol.get_info_raw('ns/id/%s' % hsh[1:]).addCallback(self._update_network_status).addErrback(log.err)
-        txtorlog.msg("NEWDESC", args)
-
     def _maybe_create_circuit(self, circ_id):
         if circ_id not in self.circuits:
             c = self.circuit_factory(self)
@@ -709,7 +702,6 @@ class TorState(object):
                  'CIRC': _circuit_update,
                  'NS': _update_network_status,
                  'NEWCONSENSUS': _update_network_status,
-                 'NEWDESC': _newdesc_update,
                  'ADDRMAP': _addr_map}
     """event_map used by add_events to map event_name -> unbound method"""
     @defer.inlineCallbacks
